==> retro_data_structures-0.26.0-py3-none-any.whl: retro_data_structures/properties/corruption/archetypes/WeaponVulnerability.py <==
# Generated File
from __future__ import annotations

import dataclasses
import struct
import typing
import typing_extensions

from retro_data_structures import json_util
from retro_data_structures.game_check import Game
from retro_data_structures.properties.base_property import BaseProperty
from retro_data_structures.properties.field_reflection import FieldReflection

if typing.TYPE_CHECKING:
    class WeaponVulnerabilityJson(typing_extensions.TypedDict):
        damage_multiplier: float
        effect: int
        ignore_radius: bool
    

_FAST_FORMAT = None
_FAST_IDS = (0x852d3871, 0x68acbd86, 0x93677da2)


@dataclasses.dataclass()
class WeaponVulnerability(BaseProperty):
    damage_multiplier: float = dataclasses.field(default=100.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0x852d3871, original_name='DamageMultiplier'
        ),
    })
    effect: int = dataclasses.field(default=0, metadata={
        'reflection': FieldReflection[int](
            int, id=0x68acbd86, original_name='Effect'
        ),
    })  # Choice
    ignore_radius: bool = dataclasses.field(default=False, metadata={
        'reflection': FieldReflection[bool](
            bool, id=0x93677da2, original_name='IgnoreRadius'
        ),
    })

    @classmethod
    def game(cls) -> Game:
        return Game.CORRUPTION

    @classmethod
    def from_stream(cls, data: typing.BinaryIO, size: int | None = None, default_override: dict | None = None) -> typing_extensions.Self:
        property_count = struct.unpack(">H", data.read(2))[0]
        if (result := cls._fast_decode(data, property_count)) is not None:
            return result

        present_fields = default_override or {}
        for _ in range(property_count):
            property_id, property_size = struct.unpack(">LH", data.read(6))
            start = data.tell()
            try:
                property_name, decoder = _property_decoder[property_id]
                present_fields[property_name] = decoder(data, property_size)
            except KeyError:
                raise RuntimeError(f"Unknown property: 0x{property_id:08x}")
            assert data.tell() - start == property_size

        return cls(**present_fields)

    @classmethod
    def _fast_decode(cls, data: typing.BinaryIO, property_count: int) -> typing_extensions.Self | None:
        if property_count != 3:
            return None
    
        global _FAST_FORMAT
        if _FAST_FORMAT is None:
            _FAST_FORMAT = struct.Struct('>LHfLHLLH?')
    
        dec = _FAST_FORMAT.unpack(data.read(27))
        assert (dec[0], dec[3], dec[6]) == _FAST_IDS
        return cls(
            dec[2],
            dec[5],
            dec[8],
        )

    def to_stream(self, data: typing.BinaryIO, default_override: dict | None = None) -> None:
        default_override = default_override or {}
        data.write(b'\x00\x03')  # 3 properties

        data.write(b'\x85-8q')  # 0x852d3871
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.damage_multiplier))

        data.write(b'h\xac\xbd\x86')  # 0x68acbd86
        data.write(b'\x00\x04')  # size
        data.write(struct.pack(">L", self.effect))

        data.write(b'\x93g}\xa2')  # 0x93677da2
        data.write(b'\x00\x01')  # size
        data.write(struct.pack('>?', self.ignore_radius))

    @classmethod
    def from_json(cls, data: json_util.JsonValue) -> typing_extensions.Self:
        json_data = typing.cast("WeaponVulnerabilityJson", data)
        return cls(
            damage_multiplier=json_data['damage_multiplier'],
            effect=json_data['effect'],
            ignore_radius=json_data['ignore_radius'],
        )

    def to_json(self) -> json_util.JsonObject:
        return {
            'damage_multiplier': self.damage_multiplier,
            'effect': self.effect,
            'ignore_radius': self.ignore_radius,
        }


def _decode_damage_multiplier(data: typing.BinaryIO, property_size: int):
    return struct.unpack('>f', data.read(4))[0]


def _decode_effect(data: typing.BinaryIO, property_size: int):
    return struct.unpack(">L", data.read(4))[0]


def _decode_ignore_radius(data: typing.BinaryIO, property_size: int):
    return struct.unpack('>?', data.read(1))[0]


_property_decoder: typing.Dict[int, typing.Tuple[str, typing.Callable[[typing.BinaryIO, int], typing.Any]]] = {
    0x852d3871: ('damage_multiplier', _decode_damage_multiplier),
    0x68acbd86: ('effect', _decode_effect),
    0x93677da2: ('ignore_radius', _decode_ignore_radius),
}

==> retro_data_structures-0.29.0-py3-none-any.whl: retro_data_structures/properties/corruption/archetypes/WeaponVulnerability.py <==
# Generated File
from __future__ import annotations

import dataclasses
import struct
import typing
import typing_extensions

from retro_data_structures import json_util
from retro_data_structures.game_check import Game
from retro_data_structures.properties.base_property import BaseProperty
from retro_data_structures.properties.field_reflection import FieldReflection

if typing.TYPE_CHECKING:
    class WeaponVulnerabilityJson(typing_extensions.TypedDict):
        damage_multiplier: float
        effect: int
        ignore_radius: bool
    

_FAST_FORMAT: struct.Struct | None = None
_FAST_IDS = (0x852d3871, 0x68acbd86, 0x93677da2)


@dataclasses.dataclass()
class WeaponVulnerability(BaseProperty):
    damage_multiplier: float = dataclasses.field(default=100.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0x852d3871, original_name='DamageMultiplier'
        ),
    })
    effect: int = dataclasses.field(default=0, metadata={
        'reflection': FieldReflection[int](
            int, id=0x68acbd86, original_name='Effect'
        ),
    })  # Choice
    ignore_radius: bool = dataclasses.field(default=False, metadata={
        'reflection': FieldReflection[bool](
            bool, id=0x93677da2, original_name='IgnoreRadius'
        ),
    })

    @classmethod
    def game(cls) -> Game:
        return Game.CORRUPTION

    @classmethod
    def from_stream(cls, data: typing.BinaryIO, size: int | None = None, default_override: dict | None = None) -> typing_extensions.Self:
        property_count = struct.unpack(">H", data.read(2))[0]
        if (result := cls._fast_decode(data, property_count)) is not None:
            return result

        present_fields = default_override or {}
        for _ in range(property_count):
            property_id, property_size = struct.unpack(">LH", data.read(6))
            start = data.tell()
            try:
                property_name, decoder = _property_decoder[property_id]
                present_fields[property_name] = decoder(data, property_size)
            except KeyError:
                raise RuntimeError(f"Unknown property: 0x{property_id:08x}")
            assert data.tell() - start == property_size

        return cls(**present_fields)

    @classmethod
    def _fast_decode(cls, data: typing.BinaryIO, property_count: int) -> typing_extensions.Self | None:
        if property_count != 3:
            return None
    
        global _FAST_FORMAT
        if _FAST_FORMAT is None:
            _FAST_FORMAT = struct.Struct('>LHfLHLLH?')
    
        dec = _FAST_FORMAT.unpack(data.read(27))
        assert (dec[0], dec[3], dec[6]) == _FAST_IDS
        return cls(
            dec[2],
            dec[5],
            dec[8],
        )

    def to_stream(self, data: typing.BinaryIO, default_override: dict | None = None) -> None:
        default_override = default_override or {}
        data.write(b'\x00\x03')  # 3 properties

        data.write(b'\x85-8q')  # 0x852d3871
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.damage_multiplier))

        data.write(b'h\xac\xbd\x86')  # 0x68acbd86
        data.write(b'\x00\x04')  # size
        data.write(struct.pack(">L", self.effect))

        data.write(b'\x93g}\xa2')  # 0x93677da2
        data.write(b'\x00\x01')  # size
        data.write(struct.pack('>?', self.ignore_radius))

    @classmethod
    def from_json(cls, data: json_util.JsonValue) -> typing_extensions.Self:
        json_data = typing.cast("WeaponVulnerabilityJson", data)
        return cls(
            damage_multiplier=json_data['damage_multiplier'],
            effect=json_data['effect'],
            ignore_radius=json_data['ignore_radius'],
        )

    def to_json(self) -> json_util.JsonObject:
        return {
            'damage_multiplier': self.damage_multiplier,
            'effect': self.effect,
            'ignore_radius': self.ignore_radius,
        }


def _decode_damage_multiplier(data: typing.BinaryIO, property_size: int) -> float:
    return struct.unpack('>f', data.read(4))[0]


def _decode_effect(data: typing.BinaryIO, property_size: int) -> int:
    return struct.unpack(">L", data.read(4))[0]


def _decode_ignore_radius(data: typing.BinaryIO, property_size: int) -> bool:
    return struct.unpack('>?', data.read(1))[0]


_property_decoder: dict[int, tuple[str, typing.Callable[[typing.BinaryIO, int], typing.Any]]] = {
    0x852d3871: ('damage_multiplier', _decode_damage_multiplier),
    0x68acbd86: ('effect', _decode_effect),
    0x93677da2: ('ignore_radius', _decode_ignore_radius),
}
